@@ -1,4 +1,5 @@
 # -*- coding: utf-8 -*-
+from cms.utils import get_cms_setting
 from django.conf import settings
 from django.core.cache import cache
 
@@ -13,23 +14,14 @@ PERMISSION_KEYS = [
 
 def get_cache_key(user, key):
     return "%s:permission:%s:%s" % (
-        settings.CMS_CACHE_PREFIX, user.username, key)
-
-def get_cache_version_key():
-    return "%s:permission:version" % settings.CMS_CACHE_PREFIX
-
-def get_cache_version():
-    version = cache.get(get_cache_version_key())
-    if version is None:
-        version = 1
-    return version
+        get_cms_setting('CACHE_PREFIX'), user.username, key)
 
 
 def get_permission_cache(user, key):
     """
     Helper for reading values from cache
     """
-    return cache.get(get_cache_key(user, key), version=get_cache_version())
+    return cache.get(get_cache_key(user, key))
 
 
 def set_permission_cache(user, key, value):
@@ -39,9 +31,7 @@ def set_permission_cache(user, key, value):
     """
     # store this key, so we can clean it when required
     cache_key = get_cache_key(user, key)
-    cache.set(cache_key, value,
-              settings.CMS_CACHE_DURATIONS['permissions'],
-              version=get_cache_version())
+    cache.set(cache_key, value, get_cms_setting('CACHE_DURATIONS')['permissions'])
 
 
 def clear_user_permission_cache(user):
@@ -49,13 +39,12 @@ def clear_user_permission_cache(user):
     Cleans permission cache for given user.
     """
     for key in PERMISSION_KEYS:
-        cache.delete(get_cache_key(user, key), version=get_cache_version())
+        cache.delete(get_cache_key(user, key))
 
 
 def clear_permission_cache():
-    version = get_cache_version()
-    if version > 1:
-        cache.incr(get_cache_version_key())
-    else:
-        cache.set(get_cache_version_key(), 2,
-                  settings.CMS_CACHE_DURATIONS['permissions'])
+    users = User.objects.filter(is_active=True)
+    for user in users:
+        for key in PERMISSION_KEYS:
+            cache_key = get_cache_key(user, key)
+            cache.delete(cache_key)
