@@ -81,8 +81,11 @@ def get_page_cache_key(lang):
 
 def _clean_many(prefix):
     keys = []
-    for lang in [language[0] for language in settings.LANGUAGES]:
-        keys.append(_get_key(prefix, lang))
+    if settings.USE_I18N:
+        for lang in [language[0] for language in settings.LANGUAGES]:
+            keys.append(_get_key(prefix, lang))
+    else:
+        keys = [_get_key(prefix, settings.LANGUAGE_CODE)]
     cache.delete_many(keys)
 
 def clean_site_choices_cache(sender, **kwargs):
