@@ -2,6 +2,7 @@
 """
 Edit Toolbar middleware
 """
+from cms.plugin_pool import plugin_pool
 from cms.cms_toolbar import CMSToolbar
 from django.http import HttpResponse
 from django.template.loader import render_to_string
@@ -9,12 +10,19 @@ from django.template.loader import render_to_string
 
 def toolbar_plugin_processor(instance, placeholder, rendered_content, original_context):
     original_context.push()
+    child_plugin_classes = []
+    if instance.get_plugin_class().allow_children:
+        instance, plugin = instance.get_plugin_instance()
+        for child_class_name in plugin.get_child_classes(placeholder, original_context['request'].current_page):
+            cls = plugin_pool.get_plugin(child_class_name)
+            child_plugin_classes.append((cls.__name__, unicode(cls.name)))
     data = {
         'instance': instance,
-        'rendered_content': rendered_content
+        'rendered_content': rendered_content,
+        'child_plugin_classes': child_plugin_classes,
     }
     original_context.update(data)
-    output = render_to_string('cms/toolbar/placeholder_wrapper.html', original_context)
+    output = render_to_string(instance.get_plugin_class().frontend_edit_template, original_context)
     original_context.pop()
     return output
 
