@@ -1,13 +1,15 @@
 # -*- coding: utf-8 -*-
+from itertools import chain
+
 from classytags.arguments import Argument, MultiValueArgument
 from classytags.core import Options, Tag
 from classytags.helpers import InclusionTag, AsTag
 from classytags.parser import Parser
 from cms.models import Page, Placeholder as PlaceholderModel
-from cms.plugin_rendering import render_plugins, render_placeholder
+from cms.plugin_rendering import render_placeholder
 from cms.plugins.utils import get_plugins, assign_plugins
-from cms.utils import get_language_from_request
-from cms.utils.moderator import get_cmsplugin_queryset, get_page_queryset
+from cms.utils import get_language_from_request, get_cms_setting
+from cms.utils.page_resolver import get_page_queryset, use_draft
 from cms.utils.placeholder import validate_placeholder_name
 from django import template
 from django.conf import settings
@@ -17,13 +19,13 @@ from django.core.mail import mail_managers
 from django.utils.html import escape
 from django.utils.safestring import mark_safe
 from django.utils.translation import ugettext_lazy as _, get_language
-from itertools import chain
 import re
 from sekizai.helpers import Watcher, get_varname
 
 
 register = template.Library()
 
+
 def get_site_id(site):
     if site:
         if isinstance(site, Site):
@@ -36,22 +38,28 @@ def get_site_id(site):
         site_id = settings.SITE_ID
     return site_id
 
+
 def has_permission(page, request):
     return page.has_change_permission(request)
+
+
 register.filter(has_permission)
 
 CLEAN_KEY_PATTERN = re.compile(r'[^a-zA-Z0-9_-]')
 
+
 def _clean_key(key):
     return CLEAN_KEY_PATTERN.sub('-', key)
 
+
 def _get_cache_key(name, page_lookup, lang, site_id):
     if isinstance(page_lookup, Page):
         page_key = str(page_lookup.pk)
     else:
         page_key = str(page_lookup)
     page_key = _clean_key(page_key)
-    return name+'__page_lookup:'+page_key+'_site:'+str(site_id)+'_lang:'+str(lang)
+    return name + '__page_lookup:' + page_key + '_site:' + str(site_id) + '_lang:' + str(lang)
+
 
 def _get_page_by_untyped_arg(page_lookup, request, site_id):
     """
@@ -75,13 +83,26 @@ def _get_page_by_untyped_arg(page_lookup, request, site_id):
         raise TypeError('The page_lookup argument can be either a Dictionary, Integer, Page, or String.')
     page_lookup.update({'site': site_id})
     try:
-        return get_page_queryset(request).get(**page_lookup)
+        if 'pk' in page_lookup:
+            page = Page.objects.all().get(**page_lookup)
+            if request and use_draft(request):
+                if page.publisher_is_draft:
+                    return page
+                else:
+                    return page.publisher_draft
+            else:
+                if page.publisher_is_draft:
+                    return page.publisher_public
+                else:
+                    return page
+        else:
+            return get_page_queryset(request).get(**page_lookup)
     except Page.DoesNotExist:
         site = Site.objects.get_current()
-        subject = _('Page not found on %(domain)s') % {'domain':site.domain}
+        subject = _('Page not found on %(domain)s') % {'domain': site.domain}
         body = _("A template tag couldn't find the page with lookup arguments `%(page_lookup)s\n`. "
-            "The URL of the request was: http://%(host)s%(path)s") \
-            % {'page_lookup': repr(page_lookup), 'host': site.domain, 'path': request.path}
+                 "The URL of the request was: http://%(host)s%(path)s") \
+               % {'page_lookup': repr(page_lookup), 'host': site.domain, 'path': request.path}
         if settings.DEBUG:
             raise Page.DoesNotExist(body)
         else:
@@ -89,6 +110,7 @@ def _get_page_by_untyped_arg(page_lookup, request, site_id):
                 mail_managers(subject, body, fail_silently=True)
             return None
 
+
 class PageUrl(InclusionTag):
     template = 'cms/content.html'
     name = 'page_url'
@@ -109,16 +131,18 @@ class PageUrl(InclusionTag):
             return {'content': ''}
         if lang is None:
             lang = get_language_from_request(request)
-        cache_key = _get_cache_key('page_url', page_lookup, lang, site_id)+'_type:absolute_url'
+        cache_key = _get_cache_key('page_url', page_lookup, lang, site_id) + '_type:absolute_url'
         url = cache.get(cache_key)
         if not url:
             page = _get_page_by_untyped_arg(page_lookup, request, site_id)
             if page:
                 url = page.get_absolute_url(language=lang)
-                cache.set(cache_key, url, settings.CMS_CACHE_DURATIONS['content'])
+                cache.set(cache_key, url, get_cms_setting('CACHE_DURATIONS')['content'])
         if url:
             return {'content': url}
         return {'content': ''}
+
+
 register.tag(PageUrl)
 
 register.tag('page_id_url', PageUrl)
@@ -126,6 +150,7 @@ register.tag('page_id_url', PageUrl)
 
 def _get_placeholder(current_page, page, context, name):
     from cms.utils.plugins import get_placeholders
+
     placeholder_cache = getattr(current_page, '_tmp_placeholders_cache', {})
     if page.pk in placeholder_cache:
         return placeholder_cache[page.pk].get(name, None)
@@ -139,6 +164,7 @@ def _get_placeholder(current_page, page, context, name):
     current_page._tmp_placeholders_cache = placeholder_cache
     return placeholder_cache[page.pk].get(name, None)
 
+
 def get_placeholder_content(context, request, current_page, name, inherit):
     edit_mode = getattr(request, 'toolbar', None) and getattr(request.toolbar, 'edit_mode')
     pages = [current_page]
@@ -156,9 +182,9 @@ def get_placeholder_content(context, request, current_page, name, inherit):
         content = render_placeholder(placeholder, context, name)
         if content:
             return content
-    # if we reach this point, we have an empty or non-existant placeholder
-    # call _get_placeholder again to get the placeholder properly rendered
-    # in frontend editing
+            # if we reach this point, we have an empty or non-existant placeholder
+            # call _get_placeholder again to get the placeholder properly rendered
+            # in frontend editing
     placeholder = _get_placeholder(current_page, current_page, context, name)
     return render_placeholder(placeholder, context, name)
 
@@ -217,6 +243,7 @@ class Placeholder(Tag):
             elif bit.isdigit():
                 width = int(bit)
                 import warnings
+
                 warnings.warn(
                     "The width parameter for the placeholder tag is deprecated.",
                     DeprecationWarning
@@ -241,9 +268,39 @@ class Placeholder(Tag):
 
     def get_name(self):
         return self.kwargs['name'].var.value.strip('"').strip("'")
+
+
 register.tag(Placeholder)
 
 
+class RenderPlugin(InclusionTag):
+    template = 'cms/content.html'
+    name = 'render_plugin'
+    options = Options(
+        Argument('plugin')
+    )
+
+    def get_context(self, context, plugin):
+        # Prepend frontedit toolbar output if applicable
+        edit = False
+        request = context['request']
+        toolbar = getattr(request, 'toolbar', None)
+        page = request.current_page
+        if toolbar.edit_mode and (not page or page.has_change_permission(request)):
+            edit = True
+        if edit:
+            from cms.middleware.toolbar import toolbar_plugin_processor
+
+            processors = (toolbar_plugin_processor,)
+        else:
+            processors = None
+
+        return {'content': plugin.render_plugin(context, processors=processors)}
+
+
+register.tag(RenderPlugin)
+
+
 class PageAttribute(AsTag):
     """
     This template node is used to output attribute from a page such
@@ -311,8 +368,11 @@ class PageAttribute(AsTag):
             func = getattr(page, "get_%s" % name)
             return escape(func(language=lang, fallback=True))
         return ''
+
+
 register.tag(PageAttribute)
 
+
 class CleanAdminListFilter(InclusionTag):
     template = 'admin/filter.html'
     name = 'clean_admin_list_filter'
@@ -330,7 +390,7 @@ class CleanAdminListFilter(InclusionTag):
             if choice['query_string'] != query_string:
                 unique_choices.append(choice)
                 query_string = choice['query_string']
-        return {'title': spec.title(), 'choices' : unique_choices}
+        return {'title': spec.title(), 'choices': unique_choices}
 
 
 def _restore_sekizai(context, changes):
@@ -386,12 +446,13 @@ def _show_placeholder_for_page(context, placeholder_name, page_lookup, lang=None
     content = render_placeholder(placeholder, context, placeholder_name)
     changes = watcher.get_changes()
     if cache_result:
-        cache.set(cache_key, {'content': content, 'sekizai': changes}, settings.CMS_CACHE_DURATIONS['content'])
+        cache.set(cache_key, {'content': content, 'sekizai': changes}, get_cms_setting('CACHE_DURATIONS')['content'])
 
     if content:
         return {'content': mark_safe(content)}
     return {'content': ''}
 
+
 class ShowPlaceholderById(InclusionTag):
     template = 'cms/content.html'
     name = 'show_placeholder_by_id'
@@ -414,20 +475,25 @@ class ShowPlaceholderById(InclusionTag):
             'lang': lang,
             'site': site
         }
+
+
 register.tag(ShowPlaceholderById)
 register.tag('show_placeholder', ShowPlaceholderById)
 
+
 class ShowUncachedPlaceholderById(ShowPlaceholderById):
     name = 'show_uncached_placeholder_by_id'
+
     def get_kwargs(self, *args, **kwargs):
         kwargs = super(ShowUncachedPlaceholderById, self).get_kwargs(*args, **kwargs)
         kwargs['cache_result'] = False
         return kwargs
+
+
 register.tag(ShowUncachedPlaceholderById)
 register.tag('show_uncached_placeholder', ShowUncachedPlaceholderById)
 
 
-
 class CMSToolbar(InclusionTag):
     template = 'cms/toolbar/toolbar.html'
     name = 'cms_toolbar'
@@ -446,4 +512,6 @@ class CMSToolbar(InclusionTag):
     def get_context(self, context):
         context['CMS_TOOLBAR_CONFIG'] = context['request'].toolbar.as_json(context)
         return context
+
+
 register.tag(CMSToolbar)
