@@ -1,38 +1,41 @@
 # -*- coding: utf-8 -*-
 from __future__ import with_statement
 from cms.api import create_page, publish_page, add_plugin
-from cms.conf.patch import post_patch_check
 from cms.exceptions import PluginAlreadyRegistered, PluginNotRegistered
 from cms.models import Page, Placeholder
 from cms.models.pluginmodel import CMSPlugin, PluginModelBase
 from cms.plugin_base import CMSPluginBase
 from cms.plugin_pool import plugin_pool
+from cms.plugins.utils import get_plugins_for_page
 from cms.plugins.file.models import File
 from cms.plugins.inherit.models import InheritPagePlaceholder
 from cms.plugins.link.forms import LinkForm
 from cms.plugins.link.models import Link
+from cms.plugins.picture.models import Picture
 from cms.plugins.text.models import Text
-from cms.plugins.text.utils import (plugin_tags_to_id_list, 
+from cms.plugins.text.utils import (plugin_tags_to_id_list,
     plugin_tags_to_admin_html)
 from cms.plugins.twitter.models import TwitterRecentEntries
 from cms.test_utils.project.pluginapp.models import Article, Section
 from cms.test_utils.project.pluginapp.plugins.manytomany_rel.models import (
     ArticlePluginModel)
-from cms.test_utils.testcases import (CMSTestCase, URL_CMS_PAGE, 
-    URL_CMS_PAGE_ADD, URL_CMS_PLUGIN_ADD, URL_CMS_PLUGIN_EDIT, URL_CMS_PAGE_CHANGE, URL_CMS_PLUGIN_REMOVE)
+from cms.test_utils.testcases import (CMSTestCase, URL_CMS_PAGE, URL_CMS_PLUGIN_MOVE,
+    URL_CMS_PAGE_ADD, URL_CMS_PLUGIN_ADD, URL_CMS_PLUGIN_EDIT, URL_CMS_PAGE_CHANGE, URL_CMS_PLUGIN_REMOVE, URL_CMS_PLUGIN_HISTORY_EDIT)
 from cms.sitemaps.cms_sitemap import CMSSitemap
 from cms.test_utils.util.context_managers import SettingsOverride
 from cms.utils.copy_plugins import copy_plugins_to
-from cms.utils import timezone
+from django.utils import timezone
 from django.conf import settings
+from django.contrib import admin
 from django.contrib.auth.models import User
-from django.core.exceptions import ImproperlyConfigured
+from django.core.exceptions import ImproperlyConfigured, ValidationError
 from django.core.files.uploadedfile import SimpleUploadedFile
 from django.core.management import call_command
 from django.forms.widgets import Media
 from django.test.testcases import TestCase
 import os
 import datetime
+from cms.utils.permissions import set_current_user
 
 
 class DumbFixturePlugin(CMSPluginBase):
@@ -40,6 +43,7 @@ class DumbFixturePlugin(CMSPluginBase):
     name = "Dumb Test Plugin. It does nothing."
     render_template = ""
     admin_preview = False
+    allow_children = True
 
     def render(self, context, instance, placeholder):
         return context
@@ -47,7 +51,7 @@ class DumbFixturePlugin(CMSPluginBase):
 class PluginsTestBaseCase(CMSTestCase):
 
     def setUp(self):
-        self.super_user = User(username="test", is_staff = True, is_active = True, is_superuser = True)
+        self.super_user = User(username="test", is_staff=True, is_active=True, is_superuser=True)
         self.super_user.set_password("test")
         self.super_user.save()
 
@@ -58,10 +62,10 @@ class PluginsTestBaseCase(CMSTestCase):
 
         self.FIRST_LANG = settings.LANGUAGES[0][0]
         self.SECOND_LANG = settings.LANGUAGES[1][0]
-        
+
         self._login_context = self.login_user_context(self.super_user)
         self._login_context.__enter__()
-    
+
     def tearDown(self):
         self._login_context.__exit__(None, None, None)
 
@@ -92,7 +96,7 @@ class PluginsTestCase(PluginsTestBaseCase):
         return created_plugin_id
 
     def _edit_text_plugin(self, plugin_id, text):
-        edit_url = "%s%s/" % (URL_CMS_PLUGIN_EDIT, plugin_id) 
+        edit_url = "%s%s/" % (URL_CMS_PLUGIN_EDIT, plugin_id)
         response = self.client.get(edit_url)
         self.assertEquals(response.status_code, 200)
         data = {
@@ -120,18 +124,44 @@ class PluginsTestCase(PluginsTestBaseCase):
             "body":"Hello World!!",
             "_cancel":True,
         }
-        response = self.client.post(URL_CMS_PAGE_ADD, data)
+        edit_url = '%s%d/' % (URL_CMS_PLUGIN_EDIT, created_plugin_id)
+        response = self.client.post(edit_url, data)
         self.assertEquals(response.status_code, 200)
         txt = Text.objects.all()[0]
         self.assertEquals("Hello World", txt.body)
 
+    def test_plugin_history_view(self):
+        """
+        Test plugin history view
+        """
+        import reversion
+        page_data = self.get_new_page_data()
+        # two versions created by simply creating the page
+        response = self.client.post(URL_CMS_PAGE_ADD, page_data)
+        page = Page.objects.all()[0]
+        page_id = int(page.pk)
+        # page version 3
+        created_plugin_id = self._create_text_plugin_on_page(page)
+        # page version 4
+        txt = self._edit_text_plugin(created_plugin_id, "Hello Foo")
+        self.assertEquals("Hello Foo", txt.body)
+        # page version 5
+        txt = self._edit_text_plugin(created_plugin_id, "Hello Bar")
+        self.assertEquals("Hello Bar", txt.body)
+        versions = [v.pk for v in reversed(reversion.get_for_object(page))]
+        history_url = '%s%d/' % (
+            URL_CMS_PLUGIN_HISTORY_EDIT % (page_id, versions[-2]),
+            created_plugin_id)
+        response = self.client.get(history_url)
+        self.assertEquals(response.status_code, 200)
+        self.assertIn('Hello Foo', response.content)
 
     def test_plugin_order(self):
         """
         Test that plugin position is saved after creation
         """
         page_en = create_page("PluginOrderPage", "col_two.html", "en",
-                              slug="page1",published=True, in_navigation=True)
+                              slug="page1", published=True, in_navigation=True)
         ph_en = page_en.placeholders.get(slot="col_left")
 
         # We check created objects and objects from the DB to be sure the position value
@@ -141,14 +171,14 @@ class PluginsTestCase(PluginsTestBaseCase):
         db_plugin_1 = CMSPlugin.objects.get(pk=text_plugin_1.pk)
         db_plugin_2 = CMSPlugin.objects.get(pk=text_plugin_2.pk)
 
-        with SettingsOverride(CMS_MODERATOR=False, CMS_PERMISSION=False):
-            self.assertEqual(text_plugin_1.position,1)
-            self.assertEqual(db_plugin_1.position,1)
-            self.assertEqual(text_plugin_2.position,2)
-            self.assertEqual(db_plugin_2.position,2)
+        with SettingsOverride(CMS_PERMISSION=False):
+            self.assertEqual(text_plugin_1.position, 1)
+            self.assertEqual(db_plugin_1.position, 1)
+            self.assertEqual(text_plugin_2.position, 2)
+            self.assertEqual(db_plugin_2.position, 2)
             ## Finally we render the placeholder to test the actual content
-            rendered_placeholder = ph_en.render(self.get_context(page_en.get_absolute_url()),None)
-            self.assertEquals(rendered_placeholder,"I'm the firstI'm the second")
+            rendered_placeholder = ph_en.render(self.get_context(page_en.get_absolute_url()), None)
+            self.assertEquals(rendered_placeholder, "I'm the firstI'm the second")
 
     def test_add_cancel_plugin(self):
         """
@@ -244,54 +274,54 @@ class PluginsTestCase(PluginsTestBaseCase):
         page_de = create_page("CopyPluginTestPage (DE)", "nav_playground.html", "de")
         ph_en = page_en.placeholders.get(slot="body")
         ph_de = page_de.placeholders.get(slot="body")
-        
+
         # add the text plugin
         text_plugin_en = add_plugin(ph_en, "TextPlugin", "en", body="Hello World")
         self.assertEquals(text_plugin_en.pk, CMSPlugin.objects.all()[0].pk)
-        
+
         # add a *nested* link plugin
         link_plugin_en = add_plugin(ph_en, "LinkPlugin", "en", target=text_plugin_en,
                                  name="A Link", url="https://www.django-cms.org")
-        
+
         # the call above to add a child makes a plugin reload required here.
         text_plugin_en = self.reload(text_plugin_en)
-        
+
         # check the relations
         self.assertEquals(text_plugin_en.get_children().count(), 1)
         self.assertEqual(link_plugin_en.parent.pk, text_plugin_en.pk)
-        
+
         # just sanity check that so far everything went well
         self.assertEqual(CMSPlugin.objects.count(), 2)
-        
+
         # copy the plugins to the german placeholder
-        copy_plugins_to(ph_en.cmsplugin_set.all(), ph_de, 'de')
-        
+        copy_plugins_to(ph_en.get_plugins(), ph_de, 'de')
+
         self.assertEqual(ph_de.cmsplugin_set.filter(parent=None).count(), 1)
         text_plugin_de = ph_de.cmsplugin_set.get(parent=None).get_plugin_instance()[0]
         self.assertEqual(text_plugin_de.get_children().count(), 1)
         link_plugin_de = text_plugin_de.get_children().get().get_plugin_instance()[0]
-        
-        
+
+
         # check we have twice as many plugins as before
         self.assertEqual(CMSPlugin.objects.count(), 4)
-        
+
         # check language plugins
         self.assertEqual(CMSPlugin.objects.filter(language='de').count(), 2)
         self.assertEqual(CMSPlugin.objects.filter(language='en').count(), 2)
-        
-        
+
+
         text_plugin_en = self.reload(text_plugin_en)
         link_plugin_en = self.reload(link_plugin_en)
-        
+
         # check the relations in english didn't change
         self.assertEquals(text_plugin_en.get_children().count(), 1)
         self.assertEqual(link_plugin_en.parent.pk, text_plugin_en.pk)
-        
+
         self.assertEqual(link_plugin_de.name, link_plugin_en.name)
         self.assertEqual(link_plugin_de.url, link_plugin_en.url)
-        
+
         self.assertEqual(text_plugin_de.body, text_plugin_en.body)
-        
+
 
     def test_remove_plugin_before_published(self):
         """
@@ -343,10 +373,12 @@ class PluginsTestCase(PluginsTestBaseCase):
 
         # there should be only 1 plugin
         self.assertEquals(CMSPlugin.objects.all().count(), 1)
+        self.assertEquals(CMSPlugin.objects.filter(placeholder__page__publisher_is_draft=True).count(), 1)
 
         # publish page
         response = self.client.post(URL_CMS_PAGE + "%d/change-status/" % page.pk, {1 :1})
         self.assertEqual(response.status_code, 200)
+        self.assertEquals(Page.objects.count(), 2)
 
         # there should now be two plugins - 1 draft, 1 public
         self.assertEquals(CMSPlugin.objects.all().count(), 2)
@@ -360,7 +392,8 @@ class PluginsTestCase(PluginsTestBaseCase):
         self.assertEquals(response.status_code, 200)
 
         # there should be no plugins
-        self.assertEquals(CMSPlugin.objects.all().count(), 0)
+        self.assertEquals(CMSPlugin.objects.all().count(), 1)
+        self.assertEquals(CMSPlugin.objects.filter(placeholder__page__publisher_is_draft=False).count(), 1)
 
     def test_remove_plugin_not_associated_to_page(self):
         """
@@ -427,46 +460,76 @@ class PluginsTestCase(PluginsTestBaseCase):
         # Let's count, to make sure we didn't remove a plugin accidentally.
         number_of_plugins_after = len(plugin_pool.get_all_plugins())
         self.assertEqual(number_of_plugins_before, number_of_plugins_after)
-                
+
     def test_inheritplugin_media(self):
         """
         Test case for InheritPagePlaceholder
         """
-        with SettingsOverride(CMS_MODERATOR=False):
-            inheritfrompage = create_page('page to inherit from',
-                                          'nav_playground.html',
-                                          'en')
-            
-            body = inheritfrompage.placeholders.get(slot="body")
-            
-            plugin = TwitterRecentEntries(
-                plugin_type='TwitterRecentEntriesPlugin',
-                placeholder=body, 
-                position=1, 
-                language=settings.LANGUAGE_CODE,
-                twitter_user='djangocms',
-            )
-            plugin.insert_at(None, position='last-child', save=True)
-            
-            page = create_page('inherit from page',
-                               'nav_playground.html',
-                               'en',
-                               published=True)
-            
-            inherited_body = page.placeholders.get(slot="body")
-                    
-            inherit_plugin = InheritPagePlaceholder(
-                plugin_type='InheritPagePlaceholderPlugin',
-                placeholder=inherited_body, 
-                position=1, 
-                language=settings.LANGUAGE_CODE,
-                from_page=inheritfrompage,
-                from_language=settings.LANGUAGE_CODE)
-            inherit_plugin.insert_at(None, position='last-child', save=True)
-            
-            self.client.logout()
-            response = self.client.get(page.get_absolute_url())
-            self.assertTrue('%scms/js/libs/jquery.tweet.js' % settings.STATIC_URL in response.content, response.content)
+        inheritfrompage = create_page('page to inherit from',
+                                      'nav_playground.html',
+                                      'en')
+
+        body = inheritfrompage.placeholders.get(slot="body")
+
+        plugin = TwitterRecentEntries(
+            plugin_type='TwitterRecentEntriesPlugin',
+            placeholder=body,
+            position=1,
+            language=settings.LANGUAGE_CODE,
+            twitter_user='djangocms',
+        )
+        plugin.insert_at(None, position='last-child', save=True)
+        inheritfrompage.publish()
+
+        page = create_page('inherit from page',
+                           'nav_playground.html',
+                           'en',
+                           published=True)
+
+        inherited_body = page.placeholders.get(slot="body")
+
+        inherit_plugin = InheritPagePlaceholder(
+            plugin_type='InheritPagePlaceholderPlugin',
+            placeholder=inherited_body,
+            position=1,
+            language=settings.LANGUAGE_CODE,
+            from_page=inheritfrompage,
+            from_language=settings.LANGUAGE_CODE)
+        inherit_plugin.insert_at(None, position='last-child', save=True)
+        page.publish()
+
+        self.client.logout()
+        response = self.client.get(page.get_absolute_url())
+        self.assertTrue('%scms/js/libs/jquery.tweet.js' % settings.STATIC_URL in response.content, response.content)
+
+    def test_inherit_plugin_with_empty_plugin(self):
+        inheritfrompage = create_page('page to inherit from',
+            'nav_playground.html',
+            'en', published=True)
+
+        body = inheritfrompage.placeholders.get(slot="body")
+        empty_plugin = CMSPlugin(
+            plugin_type='TextPlugin', # create an empty plugin
+            placeholder=body,
+            position=1,
+            language='en',
+        )
+        empty_plugin.insert_at(None, position='last-child', save=True)
+        other_page = create_page('other page', 'nav_playground.html', 'en', published=True)
+        inherited_body = other_page.placeholders.get(slot="body")
+        inherit_plugin = InheritPagePlaceholder(
+            plugin_type='InheritPagePlaceholderPlugin',
+            placeholder=inherited_body,
+            position=1,
+            language='en',
+            from_page=inheritfrompage,
+            from_language='en'
+        )
+        inherit_plugin.insert_at(None, position='last-child', save=True)
+        add_plugin(inherited_body, "TextPlugin", "en", body="foobar")
+        # this should not fail, even if there in an empty plugin
+        rendered = inherited_body.render(context=self.get_context(other_page.get_absolute_url()), width=200)
+        self.assertIn("foobar", rendered)
 
     def test_render_textplugin(self):
         # Setup
@@ -506,7 +569,7 @@ class PluginsTestCase(PluginsTestBaseCase):
         Test that copying of textplugins replaces references to copied plugins
         """
         page = create_page("page", "nav_playground.html", "en")
-        
+
         placeholder = page.placeholders.get(slot='body')
 
         plugin_base = CMSPlugin(
@@ -545,7 +608,7 @@ class PluginsTestCase(PluginsTestBaseCase):
 
         plugin.body = plugin_tags_to_admin_html(' {{ plugin_object %s }} {{ plugin_object %s }} ' % (str(plugin_ref_1.pk), str(plugin_ref_2.pk)))
         plugin.save()
-        self.assertEquals(plugin.pk, 1)
+
         page_data = self.get_new_page_data()
 
         #create 2nd language page
@@ -573,10 +636,10 @@ class PluginsTestCase(PluginsTestBaseCase):
         self.assertEquals(CMSPlugin.objects.filter(language=self.FIRST_LANG).count(), 3)
         self.assertEquals(CMSPlugin.objects.filter(language=self.SECOND_LANG).count(), 3)
         self.assertEquals(CMSPlugin.objects.count(), 6)
-
-        new_plugin = Text.objects.get(pk=6)
+        plugins = list(Text.objects.all())
+        new_plugin = plugins[-1]
         idlist = sorted(plugin_tags_to_id_list(new_plugin.body))
-        expected = sorted([4, 5])
+        expected = sorted([plugins[3].pk, plugins[4].pk])
         self.assertEquals(idlist, expected)
 
     def test_empty_plugin_is_ignored(self):
@@ -607,14 +670,67 @@ class PluginsTestCase(PluginsTestBaseCase):
         plugin = self._edit_text_plugin(plugin_id, "fnord")
 
         actual_last_modification_time = CMSSitemap().lastmod(page)
-        self.assertEqual(plugin.changed_date, actual_last_modification_time)
+        self.assertEqual(plugin.changed_date - datetime.timedelta(microseconds=plugin.changed_date.microsecond), actual_last_modification_time - datetime.timedelta(microseconds=actual_last_modification_time.microsecond))
+
+    def test_moving_plugin_to_different_placeholder(self):
+        plugin_pool.register_plugin(DumbFixturePlugin)
+        page = create_page("page", "nav_playground.html", "en", published=True)
+        plugin_data = {
+            'plugin_type': 'DumbFixturePlugin',
+            'language': settings.LANGUAGES[0][0],
+            'placeholder': page.placeholders.get(slot='body').pk,
+            }
+        response = self.client.post(URL_CMS_PLUGIN_ADD % page.pk, plugin_data)
+        self.assertEquals(response.status_code, 200)
+
+        plugin_data['parent_id'] = int(response.content)
+        del plugin_data['placeholder']
+        response = self.client.post(URL_CMS_PLUGIN_ADD % page.pk, plugin_data)
+        self.assertEquals(response.status_code, 200)
+
+        post = {
+            'plugin_id': int(response.content),
+            'placeholder': 'right-column',
+        }
+        response = self.client.post(URL_CMS_PLUGIN_MOVE % page.pk, post)
+        self.assertEquals(response.status_code, 200)
+
+        from cms.plugins.utils import build_plugin_tree
+        build_plugin_tree(page.placeholders.get(slot='right-column').get_plugins_list())
+        plugin_pool.unregister_plugin(DumbFixturePlugin)
+
+    def test_get_plugins_for_page(self):
+        page_en = create_page("PluginOrderPage", "col_two.html", "en",
+                              slug="page1", published=True, in_navigation=True)
+        ph_en = page_en.placeholders.get(slot="col_left")
+        text_plugin_1 = add_plugin(ph_en, "TextPlugin", "en", body="I'm inside an existing placeholder.")
+        # This placeholder is not in the template.
+        ph_en_not_used = page_en.placeholders.create(slot="not_used")
+        text_plugin_2 = add_plugin(ph_en_not_used, "TextPlugin", "en", body="I'm inside a non-existent placeholder.")
+        page_plugins = get_plugins_for_page(None, page_en, page_en.get_title_obj_attribute('language'))
+        db_text_plugin_1 = page_plugins.get(pk=text_plugin_1.pk)
+        self.assertRaises(CMSPlugin.DoesNotExist, page_plugins.get, pk=text_plugin_2.pk)
+        self.assertEquals(db_text_plugin_1.pk, text_plugin_1.pk)
+
+    def test_is_last_in_placeholder(self):
+        """
+        Tests that children plugins don't affect the is_last_in_placeholder plugin method.
+        """
+        page_en = create_page("PluginOrderPage", "col_two.html", "en",
+                              slug="page1", published=True, in_navigation=True)
+        ph_en = page_en.placeholders.get(slot="col_left")
+        text_plugin_1 = add_plugin(ph_en, "TextPlugin", "en", body="I'm the first")
+        text_plugin_2 = add_plugin(ph_en, "TextPlugin", "en", body="I'm the second")
+        inner_text_plugin_1 = add_plugin(ph_en, "TextPlugin", "en", body="I'm the first child of text_plugin_1")
+        text_plugin_1.cmsplugin_set.add(inner_text_plugin_1)
+        self.assertEquals(text_plugin_2.is_last_in_placeholder(), True)
 
 
 class FileSystemPluginTests(PluginsTestBaseCase):
     def setUp(self):
         super(FileSystemPluginTests, self).setUp()
         call_command('collectstatic', interactive=False, verbosity=0, link=True)
-        
+
     def tearDown(self):
         for directory in [settings.STATIC_ROOT, settings.MEDIA_ROOT]:
             for root, dirs, files in os.walk(directory, topdown=False):
@@ -627,10 +743,10 @@ class FileSystemPluginTests(PluginsTestBaseCase):
                     # Now all directories we walked...
                     os.rmdir(os.path.join(root, name))
         super(FileSystemPluginTests, self).tearDown()
-        
+
     def test_fileplugin_icon_uppercase(self):
         page = create_page('testpage', 'nav_playground.html', 'en')
-        body = page.placeholders.get(slot="body") 
+        body = page.placeholders.get(slot="body")
         plugin = File(
             plugin_type='FilePlugin',
             placeholder=body,
@@ -647,22 +763,22 @@ class FileSystemPluginTests(PluginsTestBaseCase):
 class PluginManyToManyTestCase(PluginsTestBaseCase):
 
     def setUp(self):
-        self.super_user = User(username="test", is_staff = True, is_active = True, is_superuser = True)
+        self.super_user = User(username="test", is_staff=True, is_active=True, is_superuser=True)
         self.super_user.set_password("test")
         self.super_user.save()
 
         self.slave = User(username="slave", is_staff=True, is_active=True, is_superuser=False)
         self.slave.set_password("slave")
         self.slave.save()
-        
+
         self._login_context = self.login_user_context(self.super_user)
         self._login_context.__enter__()
-    
+
         # create 3 sections
         self.sections = []
         self.section_pks = []
         for i in range(3):
-            section = Section.objects.create(name="section %s" %i)
+            section = Section.objects.create(name="section %s" % i)
             self.sections.append(section)
             self.section_pks.append(section.pk)
         self.section_count = len(self.sections)
@@ -675,9 +791,10 @@ class PluginManyToManyTestCase(PluginsTestBaseCase):
                 )
         self.FIRST_LANG = settings.LANGUAGES[0][0]
         self.SECOND_LANG = settings.LANGUAGES[1][0]
-    
+
     def test_add_plugin_with_m2m(self):
         # add a new text plugin
+        self.assertEqual(ArticlePluginModel.objects.count(), 0)
         page_data = self.get_new_page_data()
         self.client.post(URL_CMS_PAGE_ADD, page_data)
         page = Page.objects.all()[0]
@@ -705,8 +822,10 @@ class PluginManyToManyTestCase(PluginsTestBaseCase):
         self.assertEquals(self.section_count, plugin.sections.count())
 
     def test_add_plugin_with_m2m_and_publisher(self):
+        self.assertEqual(ArticlePluginModel.objects.count(), 0)
         page_data = self.get_new_page_data()
-        self.client.post(URL_CMS_PAGE_ADD, page_data)
+        response = self.client.post(URL_CMS_PAGE_ADD, page_data)
+        self.assertEqual(response.status_code, 302)
         page = Page.objects.all()[0]
         placeholder = page.placeholders.get(slot="body")
 
@@ -745,6 +864,7 @@ class PluginManyToManyTestCase(PluginsTestBaseCase):
         page = publish_page(page, self.super_user)
 
         # there should now be two plugins - 1 draft, 1 public
+        self.assertEquals(2, CMSPlugin.objects.all().count())
         self.assertEquals(2, ArticlePluginModel.objects.all().count())
 
         db_counts = [plugin.sections.count() for plugin in ArticlePluginModel.objects.all()]
@@ -754,7 +874,7 @@ class PluginManyToManyTestCase(PluginsTestBaseCase):
 
     def test_copy_plugin_with_m2m(self):
         page = create_page("page", "nav_playground.html", "en")
-        
+
         placeholder = page.placeholders.get(slot='body')
 
         plugin = ArticlePluginModel(
@@ -871,14 +991,6 @@ class PluginsMetaOptionsTests(TestCase):
         self.assertEqual(plugin._meta.db_table, 'or_another')
         self.assertEqual(plugin._meta.app_label, 'one_thing')
 
-class SekizaiTests(TestCase):
-    def test_post_patch_check(self):
-        post_patch_check()
-         
-    def test_fail(self):
-        with SettingsOverride(CMS_TEMPLATES=[('fail.html', 'fail')]):
-            self.assertRaises(ImproperlyConfigured, post_patch_check)
-
 
 class LinkPluginTestCase(PluginsTestBaseCase):
     def test_does_not_verify_existance_of_url(self):
@@ -923,30 +1035,69 @@ class NoDatabasePluginTests(TestCase):
         text = Text()
         link = Link()
         self.assertNotEqual(id(text._render_meta), id(link._render_meta))
-    
+
     def test_render_meta_does_not_leak(self):
         text = Text()
         link = Link()
-        
+
         text._render_meta.text_enabled = False
         link._render_meta.text_enabled = False
-        
+
         self.assertFalse(text._render_meta.text_enabled)
         self.assertFalse(link._render_meta.text_enabled)
-        
+
         link._render_meta.text_enabled = True
 
         self.assertFalse(text._render_meta.text_enabled)
         self.assertTrue(link._render_meta.text_enabled)
-    
+
     def test_db_table_hack(self):
         # TODO: Django tests seem to leak models from test methods, somehow
         # we should clear django.db.models.loading.app_cache in tearDown.
         plugin_class = PluginModelBase('TestPlugin', (CMSPlugin,), {'__module__': 'cms.tests.plugins'})
         self.assertEqual(plugin_class._meta.db_table, 'cmsplugin_testplugin')
-    
+
     def test_db_table_hack_with_mixin(self):
         class LeftMixin: pass
         class RightMixin: pass
         plugin_class = PluginModelBase('TestPlugin2', (LeftMixin, CMSPlugin, RightMixin), {'__module__': 'cms.tests.plugins'})
         self.assertEqual(plugin_class._meta.db_table, 'cmsplugin_testplugin2')
+
+class PicturePluginTests(PluginsTestBaseCase):
+
+    def test_link_or_page(self):
+        """Test a validator: you can enter a url or a page_link, but not both."""
+
+        page_data = self.get_new_page_data()
+        response = self.client.post(URL_CMS_PAGE_ADD, page_data)
+        page = Page.objects.all()[0]
+
+        picture = Picture(url="test")
+        # Note: don't call full_clean as it will check ALL fields - including
+        # the image, which we haven't defined. Call clean() instead which
+        # just validates the url and page_link fields.
+        picture.clean()
+
+        picture.page_link = page
+        picture.url = None
+        picture.clean()
+
+        picture.url = "test"
+        self.assertRaises(ValidationError, picture.clean)
+
+
+class SimplePluginTests(TestCase):
+    def test_simple_naming(self):
+        class MyPlugin(CMSPluginBase):
+            render_template = 'base.html'
+        self.assertEqual(MyPlugin.name, 'My Plugin')
+
+    def test_simple_context(self):
+        class MyPlugin(CMSPluginBase):
+            render_template = 'base.html'
+        plugin = MyPlugin(ArticlePluginModel, admin.site)
+        context = {}
+        out_context = plugin.render(context, 1, 2)
+        self.assertEqual(out_context['instance'], 1)
+        self.assertEqual(out_context['placeholder'], 2)
+        self.assertIs(out_context, context)
