@@ -11,11 +11,13 @@ def copy_plugins_to(plugin_list, to_placeholder, to_language = None):
         else:
             plugin_language = old_plugin.language
         # do the simple copying
-        new_plugin = old_plugin.copy_plugin(to_placeholder, plugin_language, ptree)
+        new_plugin = old_plugin.copy_plugin(to_placeholder, plugin_language, ptree) # note that ptree comes back modified
         plugins_ziplist.append((new_plugin, old_plugin))
     # this magic is needed for advanced plugins like Text Plugins that can have
     # nested plugins and need to update their content based on the new plugins.
     for new_plugin, old_plugin in plugins_ziplist:
         new_instance = new_plugin.get_plugin_instance()[0]
         if new_instance:
-            new_instance.post_copy(old_plugin, plugins_ziplist)
\ No newline at end of file
+            new_instance.post_copy(old_plugin, plugins_ziplist)
+    # returns information about originals and copies
+    return plugins_ziplist
